import { shallowMount, createLocalVue } from "@vue/test-utils";
import Vuex from "vuex";
import ContentOptions from "../../../src/components/LayoutBuilderContentOptions.vue";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const localVue = createLocalVue();
const store = Store(state);

localVue.use(Vuex);

describe("ContentOptions.vue", () => {
  describe("triggering", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = 1;
    let stub;
    let wrapper;

    beforeEach(() => {
      stub = jest.fn();
      wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId
        },
        store,
        localVue
      });

      wrapper.setMethods({
        moveContent: stub,
        addContent: stub,
        editContent: stub,
        deleteContent: stub
      });
    });

    describe("'moved'", () => {
      describe("on MoveToTopButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermovetotopbutton-stub").vm.$emit("moved");
          expect(stub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: 0
          });
        });
      });

      describe("on MoveUpButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermoveupbutton-stub").vm.$emit("moved");
          expect(stub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: indexSectionBlock - 1
          });
        });
      });

      describe("on MoveDownButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper.find("layoutbuildermovedownbutton-stub").vm.$emit("moved");
          expect(stub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: indexSectionBlock + 1
          });
        });
      });

      describe("on MoveToBottomButton", () => {
        it("should call moveContent() with the correct params", () => {
          wrapper
            .find("layoutbuildermovetobottombutton-stub")
            .vm.$emit("moved");
          expect(stub).toBeCalledWith({
            indexSection,
            indexSectionBlock,
            slotId,
            to: wrapper.vm.indexLast
          });
        });
      });
    });

    describe("'element-added'", () => {
      describe("on AddBeforeButton", () => {
        it("should call addContent() with the correct params", () => {
          wrapper
            .find("layoutbuilderaddbeforebutton-stub")
            .vm.$emit("element-added");
          expect(stub).toBeCalledWith({
            indexSection,
            slotId,
            indexSectionBlock
          });
        });
      });

      describe("on AddAfterButton", () => {
        it("should call addContent() with the correct params", () => {
          wrapper
            .find("layoutbuilderaddafterbutton-stub")
            .vm.$emit("element-added");
          expect(stub).toBeCalledWith({
            indexSection,
            slotId,
            indexSectionBlock: indexSectionBlock + 1
          });
        });
      });
    });

    describe("'edited' on EditButton", () => {
      it("should call editContent() with the correct params", () => {
        wrapper.find("layoutbuildereditbutton-stub").vm.$emit("edited");
        expect(stub).toBeCalledWith({
          indexSection,
          slotId,
          indexSectionBlock,
          newId: 9
        });
      });
    });

    describe("'deleted' on DeleteButton", () => {
      it("should call deleteContent() with the correct params", () => {
        wrapper.find("layoutbuilderdeletebutton-stub").vm.$emit("deleted");
        expect(stub).toBeCalledWith({
          indexSection,
          slotId,
          indexSectionBlock
        });
      });
    });
  });

  describe("calling indexLast()", () => {
    const indexSection = 1;
    const slotId = 1;
    const indexSectionBlock = 1;

    it("should return the index of the last content block", () => {
      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId
        },
        store,
        localVue
      });

      expect(wrapper.vm.indexLast).toBe(
        store.state.sections[indexSection].content[slotId].length - 1
      );
    });
  });

  describe("Content is the first one and not the last one", () => {
    const indexSection = 1;
    const indexSectionBlock = 0;
    const slotId = 1;
    const wrapper = shallowMount(ContentOptions, {
      propsData: {
        indexSection,
        indexSectionBlock,
        slotId
      },
      store,
      localVue
    });

    it("isFirstContent() should be true", () => {
      expect(wrapper.vm.isFirstContent).toBe(true);
    });

    it("isLastContent() should be false", () => {
      expect(wrapper.vm.isLastContent).toBe(false);
    });
  });

  describe("Content is neither the last nor the first one", () => {
    const indexSection = 1;
    const indexSectionBlock = 1;
    const slotId = 1;
    const wrapper = shallowMount(ContentOptions, {
      propsData: {
        indexSection,
        indexSectionBlock,
        slotId
      },
      store,
      localVue
    });

    it("isFirstContent() should be false", () => {
      expect(wrapper.vm.isFirstContent).toBe(false);
    });

    it("isLastContent() should be false", () => {
      expect(wrapper.vm.isLastContent).toBe(false);
    });
  });

  describe("Content is the last one and not the first one", () => {
    const indexSection = 1;
    const slotId = 1;
    const indexSectionBlock =
      store.state.sections[indexSection].content[slotId].length - 1;
    let stub;
    let wrapper;

    beforeEach(() => {
      stub = jest.fn();
      wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId
        },
        store,
        localVue
      });

      wrapper.setMethods({ moveContent: stub });
    });

    it("isFirstContent() should be false", () => {
      expect(wrapper.vm.isFirstContent).toBe(false);
    });

    it("isLastContent() should be true", () => {
      expect(wrapper.vm.isLastContent).toBe(true);
    });
  });

  describe("Content is the first one and the last one", () => {
    const indexSection = 3;
    const slotId = 1;
    const indexSectionBlock = 0;
    let stub;
    let wrapper;

    beforeEach(() => {
      stub = jest.fn();
      wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId
        },
        store,
        localVue
      });

      wrapper.setMethods({ moveContent: stub });
    });

    it("isFirstContent() should be true", () => {
      expect(wrapper.vm.isFirstContent).toBe(true);
    });

    it("isLastContent() should be true", () => {
      expect(wrapper.vm.isLastContent).toBe(true);
    });
  });

  describe("calling moveContent()", () => {
    it("should commit 'move' to the store with the correct params", () => {
      const indexSection = 1;
      const indexSectionBlock = 0;
      const slotId = 1;
      const to = 0;
      const stub = jest.fn();
      store.commit = stub;

      const wrapper = shallowMount(ContentOptions, {
        propsData: {
          indexSection,
          indexSectionBlock,
          slotId
        },
        store,
        localVue
      });

      wrapper.vm.moveContent({ indexSection, slotId, indexSectionBlock, to });

      expect(stub).toHaveBeenCalledWith("move", {
        arr: store.state.sections[indexSection].content[slotId],
        to: 0,
        index: indexSectionBlock
      });
    });
  });
});
